/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef LOGEXPO_H
#define LOGEXPO_H

// area[x, r*x] under Amp * exp( -x / Tau )

#include "QUB_BasicCurve.h"

typedef struct {
	double Amp, Tau, Base;
} LogExpoComponent;

static LogExpoComponent LOGEXPO_DEFAULTS = {1.0, 1.0, 10.0};

#define LOGEXPO_NPAR 3

static char LOGEXPO_PARAM_NAMES[][MAX_NAMELEN] = {"Amp", "Tau", "LogBase"};

class QUB_Curve_LogExpo : public QUB_BasicCurve<LogExpoComponent, LOGEXPO_NPAR>
{
public:
	QUB_Curve_LogExpo(const char *name, IQUB_Curve *baselineCurve)
		: QUB_BasicCurve<LogExpoComponent, LOGEXPO_NPAR>(
				name,
				"area[x, r*x] under Amp * exp( -x / Tau ) with LogBase(x) binning",
				&(LOGEXPO_PARAM_NAMES[0][0]),
				LOGEXPO_DEFAULTS,
				baselineCurve )
	{
		setParamConst(2, 1); // const log-base
	}
	double calcR(const double *xx, int n, double Base)
	{
		if (n < 2)
			return 1.0;
		double mindx = xx[1] - xx[0];
		for (int i=2; i<n; ++i)
			if ( (xx[i] - xx[i-1]) < mindx )
				mindx = xx[i] - xx[i-1];
		return exp( mindx * log(Base) );
	}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double Amp = components[c].Amp;
		double Tau = components[c].Tau;
		double Base = components[c].Base;
		double *xx = data->xx;
		int n = data->n;
		double r = calcR(xx, n, Base);
		for (int i=0; i<n; ++i)
			yy[i] += Amp * Tau * (exp( - pow(Base,xx[i]) / Tau ) - exp( - pow(Base,xx[i]) * r / Tau ));
	}
};

// make your own factory function with this signature,
// returning a new instance of your class:

// vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
IQUB_Curve* __stdcall CreateLogExpo();
IQUB_Curve* __stdcall CreateLogExpoLinear();
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^



#endif

